"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import ConfigParser, os, copy, config, os.path, stat, shutil, sys, signal, base64

_TG_CONFIG_VERSION=12
REAPING=0

class Point2PlayError(Exception):
    """Base class for Point2Play exceptions"""
    def __init__(self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

try:
    import tests
except:
    raise Point2PlayError( "Unabled to load test case utilities" )

class badConfigFile(Point2PlayError):
    """Raised when the given config file is missing information"""

class gameAlreadyExists(Point2PlayError):
    """Raised when trying to add a game that already exists"""

class gameDoesNotExist(Point2PlayError):
    """Raised when unable to find given game name"""

class programDoesNotExist(Point2PlayError):
    """Raised when unable to find a program in a certain game"""

class executableAlreadyExists(Point2PlayError):
    """Raised when trying to add an executable that already exists"""

class winexNotFound(Point2PlayError):
    """Raised when winex binary could not be found to run application"""

class badWinexVersion(Point2PlayError):
    """Raised when winex version can't be found"""

class renameGameFailed(Point2PlayError):
    """Raised when unable to rename game"""

class pathError(Exception):
    """Raised when _{unix|windows}_to_{windows|unix}_path fails"""
    def __init__( self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

def _unix_to_windows_path( path, root, drive_letter = "C" ):
    drive_letter = drive_letter.upper( )
    if len( drive_letter ) != 1:
        raise pathError( "Drive letter not 1 letter" )
    if drive_letter < "A" or drive_letter > "Z":
        raise pathError( "Drive letter not A-Z" )
    temp = path.replace( root, drive_letter + ":" )
    new_path = temp.replace( "/", "\\" )
    return new_path

def _windows_to_unix_path( path, root, drive_letter = "C" ):
    drive_letter = drive_letter.upper()
    if len( drive_letter ) != 1:
        raise pathError( "Drive letter not 1 letter" )
    if drive_letter < "A" or drive_letter > "Z":
        raise pathError( "Drive letter not A-Z" )
    temp = path.replace( drive_letter + ":", root )
    new_path = temp.replace( "\\", "/" )
    return new_path

class Point2Play:
    def __init__( self, filename ):
        self.config_file_name = filename
        self.debug = 0
        config_file = ConfigParser.ConfigParser()
        config_file.readfp( file( filename, "r" ) )
        try:
            self.config = { "gamedir": config_file.get( "transgaming", "gamedir" ) }
            if self.config["gamedir"][ len( self.config["gamedir"]) - 1 ] != '/':
                self.config["gamedir"] = self.config["gamedir"] + "/"
            self.gamedir = self.config["gamedir"].replace( "$HOME", os.environ["HOME"] )
        except ConfigParser.NoOptionError, e:
            raise badConfigFile( "Missing options" + e.value )
        except ConfigParser.NoSectionError, e:
            raise badConfigFile( "Missing [transgaming] section" )

        self.versions = []
        try:
            versions = config_file.options("winex")
            for ver in versions:
                if ver == "default":
                    self.default_winex = config_file.get("winex","default")
                else:
                    self.versions.append( { "version": ver, "path": config_file.get("winex", ver ) } )
        except:
            pass

        try:
            self.config["accept_license"] = config_file.get( "transgaming", "accept_license" )
        except:
            self.config["accept_license"] = 0

        try:
            self.config["username"] = config_file.get("transgaming","username")
            try:
                password = config_file.get("transgaming","password")
                self.config["password"] = base64.decodestring( password )
            except:
                self.config["password"] = ""
        except:
            self.config["username"] = ""
            self.config["password"] = ""

        try:
            self.config["proxy_check"] = config_file.getint("transgaming","proxy_check")
        except:
            self.config["proxy_check"] = 0

        try:
            self.config["proxy_host"] = config_file.get("transgaming","proxy_host")
        except:
            self.config["proxy_host"] = ""

        try:
            self.config["proxy_port"] = self.config["proxy_port"] = config_file.getint("transgaming","proxy_port")
        except:
            self.config["proxy_port"] = 3128
        try:
            game_dir_list = os.listdir( self.gamedir )
        except:
            raise badConfigFile( "gamedir option points to bad directory" )

        """Only use the directories"""
        game_list = None
        for dir_entry in game_dir_list:
            if dir_entry[0] == ".":
                continue
            dir_name = self.gamedir + dir_entry
            if os.path.isdir( dir_name ):
                executables = []
                try:
                    game_config_file = file( dir_name + "/games.ini" )
                    game_config = ConfigParser.ConfigParser()
                    game_config.readfp( game_config_file )
                    for program in game_config.sections():
                        try:
                            path = game_config.get( program, "path" )
                            icon = game_config.get( program, "icon" )
                            workdir = game_config.get( program, "workdir" )
                        except:
                            print "Missing information"
                            continue
                        executables.append( { "Name": program,
                                              "Path": path,
                                              "Icon": icon,
                                              "Workdir": workdir } )
                except:
                    pass
                if game_list != None:
                    game_list[dir_entry] = executables
                else:
                    game_list = { dir_entry: executables }
        self._games = game_list
        try:
            self.tests = tests.tests()
        except tests.noTestsFound:
            pass

    def games( self ):
        if self._games:
            tup = self._games.keys()
            li = list(tup)
            li.sort()
            return tuple(li)
        else:
            return ( )

    def get_winex_path( self, version ):
        if not version or version == "default":
            version = self.default_winex
        for ver in self.versions:
            if ver["version"] == version:
                return ver["path"]
        raise badWinexVersion( version + " isn't available" )

    def run_regapi( self, path, reg_filename, version=None ):
        winexdir = self.get_winex_path( version )
        pid = os.fork()
        if pid:
            (wpid, retval) = os.waitpid( pid, 0 )
            return retval
        else:
            """We are the child"""
            try:
                os.close(0)
                os.close(1)
                os.close(2)
                os.open( reg_filename, os.O_RDONLY )
                os.open( "/dev/null", os.O_WRONLY )
                os.open( "/dev/null", os.O_WRONLY )
                new_env = copy.copy( os.environ )
                regapi = winexdir + "/bin/regapi"
                new_env["LD_LIBRARY_PATH"] = winexdir + "/lib:" + winexdir + "/bin"
                new_env["PATH"] = winexdir + "/bin:" + os.environ["PATH"]
                new_env["WINE_LOADER"] = "winex"
                new_env["__MSVCRT_HEAP_SELECT"] = "__GLOBAL_HEAP_SELECTED,2"
                new_env["WINEPREFIX"]=path
                os.execve( regapi, ( "regapi", "--", "setValue", "-force" ), new_env )
                sys.exit(1)
            except:
                sys.exit(1)

    def new_game( self, game, version = None ):
        if self._games: # Do we have any games?
            if self._games.has_key(game):
                return 1
        winexdir = self.get_winex_path( version )
        path = self.gamedir + game + "/"
        new_env = copy.copy( os.environ )
        new_env["WINEXINSTALLDIR"] = winexdir
        new_env["WINEPREFIX"] = path
        exec_file = os.path.join( winexdir, "bin", "winex3" )
        retval = os.spawnle( os.P_WAIT, exec_file, "winex3", new_env )
        if os.path.isfile( os.path.join( path, "config" ) ):
            #print "Updating config file"
            #"""Copy the default tree"""
            #shutil.copytree( winexdir + "/.transgaming", path )
            #"""Link .so"""
            #os.symlink( winexdir + "/bin/regsvr32", path + "c_drive/windows/system32/regsvr32" )
            #os.symlink( winexdir + "/bin/regsvr32.so", path + "c_drive/windows/system32/regsvr32.so" )
            config_file = config.config_file( path + "config" )
            config_file.set_option( "Drive C", "Path", path + "c_drive" )
            config_file.write( path + "config" )
            #if self.run_regapi( path, winexdir + "/update.reg" ):
            #    shutil.rmtree( path )
            #    return 2
            #else:
            if self._games: # Do we have any games?
                 self._games[game] = []
            else:
                 self._games = { game: [] }
            return 0
        else:
            shutil.rmtree( path )
            return 1

    def rename_game( self, game, new_name ):
        if self._games:
            if self._games.has_key(game):
                if self._games.has_key( new_name ):
                    raise renameGameFailed( "New name already exists" )
                src_name = "%s/%s" % ( self.gamedir, game )
                dst_name = "%s/%s" % ( self.gamedir, new_name )
                os.rename( src_name, dst_name )
                self._games[new_name] = []
                for exe in self._games[game]:
                    self._games[new_name].append( { "Path": exe["Path"].replace( game, new_name, 1 ),
                                                    "Icon": exe["Icon"].replace( game, new_name, 1 ),
                                                    "Workdir": exe["Workdir"].replace( game, new_name, 1 ),
                                                    "Name": exe["Name"] } )
                #print self._games[new_name]
                del self._games[game]
                self.write_programs( new_name )
            else:
                raise renameGameFailed( "Game does not exist" )
        else:
            raise renameGameFailed( "No games" )

    def remove_game( self, game ):
        """Remove a game and delete it's tree"""
        if self._games:
            if self._games.has_key(game):
                del self._games[game]
                shutil.rmtree( self.gamedir + game )

    def new_program( self, game, name, path, icon, workdir ):
        """Add a new exe to a game"""
        if self._games: # Do we have any games
            if self._games.has_key( game ): # Do we have this game?
                for exe in self._games[game]:
                    if exe["Name"] == name or exe["Path"] == path:
                        raise executableAlreadyExists( name + " already exists in game " + game )
                self._games[game].append( { "Path": path, "Name": name, "Icon": icon, "Workdir": workdir } )
            else:
                raise gameDoesNotExist( game + " does not exist" )
        else:
            raise gameDoesNotExist( "No games exists" )

    def remove_program( self, game, name ):
        """Remove an exe from the game"""
        #print "Removing %s from %s" % ( game, name )
        if self._games: # Do we have any games?
            if self._games.has_key( game ):
                for i in range( 0, len( self._games[game] ) ):
                    if self._games[game][i]["Name"] == name:
                        print i, self._games[game][i]
                        del self._games[game][i]
                        return
                raise programDoesNotExist( name + " is not present in " + game )
            else:
                raise gameDoesNotExist( game + " does not exist" )
        else:
            raise gameDoesNotExist( "No games exists" )

    def winex( self, game, executable, workdir = None, install_file = None, version = None, install_ops = None ):
        """
        Function called when we wish to run an executable in a specific
        game.
        """

        path = self.gamedir + game + "/"
        game_conf = config.config_file( path + "config" )
        exe = os.path.basename( executable )
        # Which version of WineX for this app
        if not version:
            version = game_conf.get_option( "transgaming", "version", exe )

        winexdir = self.get_winex_path( version )
        print "winexdir is " + winexdir

        new_env = copy.copy( os.environ )
        run_winex = os.path.join( winexdir, "bin", "winex3" )
        #new_env["LD_LIBRARY_PATH"] = winexdir + "/lib:" + winexdir + "/bin"
        #new_env["PATH"] = winexdir + "/bin:" + os.environ["PATH"] 
        #new_env["WINE_LOADER"] = "winex"
        #new_env["__MSVCRT_HEAP_SELECT"] = "__GLOBAL_HEAP_SELECTED,2"
        new_env["WINEXINSTALLDIR"]=winexdir
        new_env["WINEPREFIX"]=path
        if new_env.has_key("LD_PRELOAD"):
            del new_env["LD_PRELOAD"]

        if install_file:
            new_env["WINEX_INSTALL_FILE"] = install_file

        #try:
        #    fp = file( path + "tg_config_version" )
        #    tg_config_version = int( fp.readline().strip() )
        #    fp.close()
        #except:
        #    tg_config_version=1

        #if tg_config_version < _TG_CONFIG_VERSION:
        #    print "Updating TransGaming config and registry info"
        #    self.update_winex( game )

        debug   = game_conf.get_option( "transgaming", "debugmsg", exe )
        cmdline = game_conf.get_option( "transgaming", "cmdline", exe )
        options = [ os.path.basename( run_winex ) ]
        if not debug or len( debug ) == 0:
            debug = "-all"
        options.append( "--debugmsg" )
        options.append( debug )
        if install_ops:
            if install_ops[1]:
                options.append( "-bigexe" )
            if install_ops[0]:
                options.append( "-use-dos-cwd" )
                options.append( os.path.dirname( executable ) )
        else:
            bigexe = game_conf.get_option( "transgaming", "bigexe", exe )
            if bigexe and bigexe == "Y":
                options.append( "-bigexe" )
            doscwd = game_conf.get_option( "transgaming", "doscwd", exe )
            if doscwd and len( doscwd ):
                options.append( "-use-dos-cwd" )
                options.append( doscwd )
        options.append( "--" )
        options.append( executable )
        if cmdline and len( cmdline ) > 0:
            map( options.append, cmdline.split( " " ) )
        if self.debug:
            for k, v in new_env.items():
                print "%s=%s" % ( k, v )
            print "Execing: %s %s" % ( run_winex, options )
        if not workdir:
            workdir = _windows_to_unix_path( os.path.dirname( executable ), path+"/c_drive" )
        saved_dir = os.getcwd()
        os.chdir( workdir )
        os.spawnve( os.P_NOWAIT, run_winex, options, new_env )
        if workdir:
            os.chdir( saved_dir )
        
    def write( self, filename=None ):
        """Save the config file"""
        if not filename:
            filename = self.config_file_name
        fp = file( filename, "w+" ) # Overwrite
        fp.write( "[transgaming]\n" ) # First section
        fp.write( "gamedir=" + self.config["gamedir"] + "\n" )
        fp.write( "username=" + self.config["username"] + "\n" )
        fp.write( "password=" )
        if len( self.config["password"] ):
            fp.write( base64.encodestring( self.config["password"] ) )
        else:
            fp.write( "\n" )
        fp.write( "proxy_check=" + str(self.config["proxy_check"]) + "\n" )
        fp.write( "proxy_host=" + self.config["proxy_host"] + "\n" )
        fp.write( "proxy_post=" + str(self.config["proxy_port"]) + "\n" )
        if self.config["accept_license"]:
            fp.write( "accept_license=1\n" )
        fp.write( "\n" )
        fp.write( "[winex]\n")
        fp.write( "default=" + self.default_winex + "\n" )
        for ver in self.versions:
            fp.write( ver["version"] + "=" + ver["path"] + "\n" )
        fp.close()

    def write_programs( self, game ):
        """Save config for that game"""
        if self._games:
            if self._games.has_key(game):
                game_filename = self.gamedir + game + "/games.ini"
                fp = file( game_filename, "w+" )
                for i in self._games[game]:
                    fp.write( "[" + i["Name"] + "]\n" )
                    fp.write( "path=" + i["Path"] + "\n" )
                    fp.write( "icon=" + i["Icon"] + "\n" )
                    fp.write( "workdir=" + i["Workdir"] + "\n" )
                fp.close()

    def update_winex( self, game, version = None ):
        if not version:
            version = self.default_winex
        for ver in self.versions:
            if ver["version"] == version:
                winexdir = ver["path"]
        path = self.gamedir + game + "/"
        os.rename( path + "config", path + "config.bak" )
        shutil.copy( winexdir + "/.transgaming/config", path + "config" )
        shutil.copy( winexdir + "/.transgaming/dyndata.reg", path + "dyndata.reg" )

        self.run_regapi( game, winexdir + "update.reg" )
        # Create My Documents
        # Skipped
        shutil.copy( winexdir + ".transgaming/c_drive/windows/system.ini", path + "c_drive/windows/system.ini" )
        for i in ( "ddraw.dll", "dinput.dll", "dinput8.dll", "d3d8.dll", "dmusic.dll", "d3drm.dll" ):
            shutil.copy( winexdir + "/.transgaming/c_drive/windows/system32/" + i, path + "c_drive/windows/system32/" + i )
        shutil.copy( winexdir + "/.transgaming/tg_config_version", path + "tg_config_version" )
