"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import sys

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

class Point2PlayError(Exception):
    def __init__( self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

try:
    import gtk, gobject
except:
    raise Point2PlayError( "Unable to load GTK2 Python bindings (%s)" % str( sys.exc_info()[1] ) )

try:
    import config_gui, install_gui, winex_get_gui, winex_get, preferences_gui, p2p_diag, license
except:
    raise Point2PlayError( "Unable to load GUI components (%s)" % str( sys.exc_info()[1] ) )

import Point2Play

try:
    import fstab, mount
except:
    raise Point2PlayError( "Unable to load fstab/cdrom utilities" )

import tempfile, os, select, signal, copy, shutil, socket

PROGRAM_ICON_COL = 0
PROGRAM_NAME_COL = 1
PROGRAM_GAME_COL = 2
PROGRAM_DIR_COL  = 3

ICON_WIDTH  = 32
ICON_HEIGHT = 32

GAME_RENAMED = 1
GAME_NOT_RENAMED = 0

def kill_wine_timeout( procs, *args ):
    for proc in procs:
        try:
            os.kill( proc, signal.SIGKILL )
        except:
            pass

class PasswordDialog( gtk.Dialog ):
    def __init__(self):
        gtk.Dialog.__init__(self)
        self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        vbox = self.get_children()[0]
        vbox.add( gtk.Label( "Password" ) )
        self.entry = gtk.Entry()
        self.entry.set_visibility( 0 )
        vbox.add( self.entry )
        vbox.show_all()

    def run(self):
        resp = gtk.Dialog.run(self)
        if resp == gtk.RESPONSE_OK:
            return self.entry.get_text()
        else:
            return None

class Point2Play_gui(gtk.Window):
    def __init__( self, Point2Play ):
        gtk.Window.__init__( self )
        self.Point2Play = Point2Play # This is the data of the Point2Play
        if not Point2Play.config["accept_license"]:
            try:
                lic_file = os.environ["P2PPATH"] + "/LICENSE"
            except:
                lic_file = "LICENSE"
            lic_fp = file( lic_file, "r" )
            lic_diag = license.LicenseDialog( lic_fp )
            resp = lic_diag.run()
            lic_diag.destroy()
            if resp == license.LICENSE_DECLINE:
                raise license.licenseError("License not accepted.")
            self.Point2Play.config["accept_license"] = 1
            self.Point2Play.write()
        self.monitor = None
        self.REAPING = 0
        self.config = copy.copy( Point2Play.config )
        menu_items = ( ( "/_File", "<alt>F", None, 0, "<Branch>" ),
                       ( "/_File/Play", None, self.play_cb, 0, None ),
                       ( "/_File/_Install", "<control>I", self.install_menu_cb, 0, None ),
                       ( "/_File/_Remove",  "<control>R", self.remove_menu_cb,  0, None ),
                       ( "/_File/Rename", None, self.rename_game_cb, 0, None ),
                       ( "/_File/Exit", "<control>Q", gtk.mainquit, 0, None ),
                       ( "/_Tools", None, None, 0, "<Branch>" ),
                       # ( "/_Tools/Add\/Remove _WineX", "<control>W", self.winex_cb, 0, None ),
                       ( "/_Tools/Icons", None, None, 0, "<Branch>" ),
                       ( "/_Tools/Icons/Add", None, self.add_icon_cb, 0, None ),
                       ( "/_Tools/Icons/Remove",  None, self.remove_icon_cb, 0, None ),
                       ( "/_Tools/(_Un)Mount CD", "<control>U", self.mount_cb, 0, None ),
                       ( "/_Tools/Stop Wine", "<control>S", self.kill_wine_cb, 0, None ),
                       # ( "/_Tools/_Preferences", "<control>P", self.prefs_menu_cb, 0, None ),
                       ( "/_Help", None, None, 0, "<Branch>" ),
                       ( "/_Help/_About", "<control>A", self.about_menu_cb, 0, None ),
                       ( "/_Help/Contact _TransGaming", "<control>T", self.contact_menu_cb, 0, None ),
                       ( "/_Help/_Support", "<control>S", self.support_menu_cb, 0, None ) ) 

        accel_group = gtk.AccelGroup()
        self.set_title( "Point2Play" )
        self.set_size_request( 610, 430 )
        self.set_resizable( 0 )
        self.set_name( "main_win" )
        self.widgets = { "menu_items": gtk.ItemFactory( gtk.MenuBar, "<main>", accel_group ) }
        self.widgets["menu_items"].create_items( menu_items )
        self.add_accel_group( accel_group )

        # Make Play A Game insensitive

        self.widgets["menu_items"].get_item( "/File/Play" ).set_sensitive( 0 )
        # self.widgets["menu_items"].get_item( "/Tools/Preferences" ).set_sensitive( 0 )
        self.widgets["menu_items"].get_item( "/Tools/Icons/Add" ).set_sensitive( 0 )
        self.widgets["menu_items"].get_item( "/Tools/Icons/Remove" ).set_sensitive( 0 )

        vbox = gtk.VBox() # Create a vbox container for main window
        self.add( vbox ) # Add to main window
        vbox.pack_start( self.widgets["menu_items"].get_widget( "<main>" ), gtk.FALSE, gtk.FALSE ) # Add menubar

        self.widgets["notebook"] = gtk.Notebook() # Create notebook for mainwin
        vbox.add( self.widgets["notebook" ] ) # Add to vbox

        hbox = gtk.HBox()
        self.widgets["game_view"] = gtk.TreeView( self.game_model( 1 ) )
        self.widgets["program_view"] = gtk.TreeView( gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING ) )
        """Connect changed button"""
        self.widgets["game_view"].get_selection().connect( 'changed', self.game_select_cb )
        self.widgets["game_view"].get_selection().set_mode( 'single' )
        self.widgets["program_view"].get_selection().connect( 'changed', self.program_select_cb )

        # Now we attach rendering widgets to the game tree
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( "Game", cell, text=0 )
        self.widgets["game_view"].append_column( column )
        self.widgets["game_view"].set_headers_visible( 0 )

        # Now we attach rendering widgets to the program tree
        cell = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn( "", cell, pixbuf=PROGRAM_ICON_COL )
        self.widgets["program_view"].append_column( column )
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( "Program", cell, text=PROGRAM_NAME_COL )
        self.widgets["program_view"].append_column( column )
        self.widgets["program_view"].set_headers_visible( 0 )

        # Create the button box
        button_vbox = gtk.VBox()
        for i in ( ( "play_button", "Play" ), ( "configure_button", "Configure" ), ( "mount_button", "(Un)Mount" ), ( "install_button", "Install" ), ( "remove_button", "Remove" ) ):
            self.widgets[i[0]] = gtk.Button(i[1])
            self.widgets[i[0]].set_name( "round_button" )

        # Add buttons to the button box
        for i in ( "play_button", "configure_button", "mount_button", "install_button", "remove_button" ):
            button_vbox.pack_start( self.widgets[i], gtk.FALSE, gtk.FALSE )

        # Make these buttons insensitve
        self.widgets["play_button"].set_sensitive( 0 )
        self.widgets["configure_button"].set_sensitive( 0 )

        # Connect buttons
        self.widgets["play_button"].connect( 'clicked', self.play_cb )
        self.widgets["configure_button"].connect( 'clicked', self.configure_button_cb )
        self.widgets["mount_button"].connect( 'clicked', self.mount_cb )
        self.widgets["remove_button"].connect( 'clicked', self.remove_menu_cb )
        self.widgets["install_button"].connect( 'clicked', self.install_menu_cb )

        # Put the tree in scrollable windows
        sw = gtk.ScrolledWindow()
        sw.add( self.widgets["game_view"] )
        sw.set_size_request( 200, 300 )
        hbox.pack_start( sw )

        sw = gtk.ScrolledWindow()
        sw.add( self.widgets["program_view"] )
        sw.set_size_request( 200, 300 )
        hbox.add( sw )

        hbox.pack_start( button_vbox, gtk.FALSE, gtk.FALSE )

        self.widgets["notebook"].append_page( hbox, gtk.Label( "Main" ) )

        """Build test page"""
        hbox_major = gtk.HBox()
        hbox = gtk.HBox(True)
        self.widgets["tests_testvbox"] = gtk.VBox()
        button_vbox = gtk.VBox()

        self.widgets["tests_testlist"] = gtk.TreeView(self.Point2Play.tests.getTestListGTKModel())
        # Now we attach rendering widgets to the game tree
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( "Test", cell, text=0 )
        self.widgets["tests_testlist"].append_column( column )
        self.widgets["tests_testlist"].set_headers_visible( 0 )

        self.widgets["tests_testlist"].connect( 'cursor_changed', self.tests_selection_cb )

        # Create a drawing area for opengl test
        self.widgets["tests_drawarea"] = gtk.DrawingArea()
        self.widgets["tests_drawarea"].set_size_request(100, 100)

        # Add buttons
        self.widgets["tests_run_button"] = gtk.Button( "Run Test" )
        self.widgets["tests_run_button"].set_name( "round_button" )
        self.widgets["tests_run_button"].connect( 'clicked', self.tests_run_cb )
        button_vbox.pack_start( self.widgets["tests_run_button"], gtk.FALSE, gtk.FALSE)

        # Put test list in a window
        sw = gtk.ScrolledWindow()
        sw.add( self.widgets["tests_testlist"] )
        sw.set_size_request( 150, 200 )
        hbox.pack_start( sw )

        # We create a "vbox" for the draw area to add / remove tests
        self.widgets["test_area_box"] = gtk.VBox()

        self.widgets["test_area_labels"] = []
        self.widgets["test_area_label_boxs"] = []

        self.widgets["test_area_table"] = gtk.Table(0, 2, False)

        self.widgets["test_area_table"].set_col_spacings(2)
        self.widgets["test_area_table"].set_row_spacings(2)

        self.widgets["test_description"] = gtk.Label("")
        self.widgets["test_description"].hide()
        self.widgets["test_description"].set_line_wrap(True)

        self.widgets["test_area_box"].pack_start( self.widgets["test_area_table"], False, False, 0 )
        self.widgets["test_area_box"].pack_start( self.widgets["test_description"], False, False, 0 )
        self.widgets["test_area_box"].add( self.widgets["tests_drawarea"] )

        # the draw window
        self.widgets["tests_testvbox"].pack_start( self.widgets["test_area_box"] )

        # test window
        hbox.pack_start(self.widgets["tests_testvbox"])

        # and finish it off
        hbox_major.pack_start(hbox, gtk.TRUE, gtk.TRUE, 5)
        hbox_major.pack_start( button_vbox, gtk.FALSE, gtk.FALSE )
        self.widgets["notebook"].append_page( hbox_major, gtk.Label( "System\nTests" ) )

        """Build update page"""
        vbox = gtk.VBox()
        hbox_username = gtk.HBox()
        table = gtk.Table( 2, 6 )
        table.attach( gtk.Label( "Username" ), 0, 1, 0, 1 )
        table.attach( gtk.Label( "Password" ), 0, 1, 1, 2 )

        for i in ( "username", "password", "proxy_host", "proxy_port" ):
            self.widgets[i] = gtk.Entry()
            self.widgets[i].connect( 'changed', self.text_update_cb, i )
        self.widgets["password"].set_visibility( 0 )
        self.widgets["proxy_check"] = gtk.CheckButton("HTTP Proxy")
        self.widgets["proxy_check"].connect('toggled', self.proxy_check_toggled )
        table.attach( self.widgets["username"], 1, 2, 0, 1 )
        table.attach( self.widgets["password"], 1, 2, 1, 2 )

        self.widgets["get_latest"] = gtk.Button( "Get Latest Version" )
        self.widgets["get_latest"].set_name( "round_button" )
        self.widgets["get_latest"].connect( 'clicked', self.get_latest_cb )

        table.attach( self.widgets["get_latest"], 0, 1, 2, 3 )
        table.attach( self.widgets["proxy_check"], 0, 1, 3, 4 )
        table.attach( gtk.Label( "Proxy hostname" ), 0, 1, 4, 5 )
        table.attach( gtk.Label( "Proxy Port" ), 0, 1, 5, 6 )
        table.attach( self.widgets["proxy_host"], 1, 2, 4, 5 )
        table.attach( self.widgets["proxy_port"], 1, 2, 5, 6 )
        # table.attach( gtk.Label( "Game Directory"), 0, 1, 4, 5 )
        # table.attach( self.widgets["gamedir"], 1, 2, 4, 5 )
        self.update_config_widgets()

        hbox_username.pack_start( table, gtk.FALSE, gtk.FALSE )
        description_label = gtk.Label( "The Point2Play Update panel allows you to\ndownload and install WineX within the Point2Play\nenvironment.  Simply enter your TransGaming\naccount username and password, and click\n'Get Latest Version' to ensure that you have the\nmost recent version of WineX on your system.\nNote that an installation of a standard WineX\npackage is seperate from a Point2Play\nWineX installation." )
        description_label.set_line_wrap( 1 )
        hbox_username.add( description_label )
        vbox.pack_start( hbox_username, gtk.FALSE, gtk.FALSE )

        self.widgets["notebook"].append_page( vbox, gtk.Label( "Update" ) )

        self.show_all()
        self.widgets["tests_drawarea"].hide()
        while gtk.events_pending():
            gtk.main_iteration()
        cdroms = fstab.fstab()
        self.cdroms = cdroms.getCdromList()

        if not len( self.cdroms ):
            self.widgets["mount_button"].set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/Tools/(Un)Mount CD" ).set_sensitive( 0 )
        elif len( self.cdroms ) > 1:
            self.CDDialog = mount.CDDialog( self.cdroms )

        if not len( self.Point2Play.versions ):
            self.widgets["menu_items"].get_item( "/File/Install" ).set_sensitive( 0 )
            self.widgets["install_button"].set_sensitive( 0 )
            # Insert message here for first time run
        if not len( self.Point2Play.games() ):
            self.widgets["remove_button"].set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/File/Remove" ).set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/File/Rename" ).set_sensitive( 0 )

    def text_update_cb( self, widget, name ):
        if not self.update_config:
            if name == "proxy_port":
                try:
                    self.config[name] = int( widget.get_text() )
                except:
                    pass
            else:
                if name == "username":
                    if len( widget.get_text() ):
                        self.widgets["get_latest"].set_sensitive( 1 )
                    else:
                        self.widgets["get_latest"].set_sensitive( 0 )
                self.config[name] = widget.get_text()

    def get_latest_cb( self, widget, *args ):
        self.Point2Play.config = copy.copy( self.config )
        self.Point2Play.write()
        if not len( self.Point2Play.config["password"] ):
            pass_diag = PasswordDialog()
            pass_diag.set_transient_for( self )
            password = pass_diag.run()
            pass_diag.destroy()
            if not password or len( password ) == 0:
                return
        else:
            password = self.Point2Play.config["password"]
        if self.Point2Play.config["proxy_check"]:
            os.environ["http_proxy"] = "http://%s:%d/" % ( self.Point2Play.config["proxy_host"], self.Point2Play.config["proxy_port"] )
        try:
            winex_get.login_transgaming( self.Point2Play.config["username"], password )
        except socket.gaierror, e:
            err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to connect to server\n%s" % e[1] )
            err_diag.run()
            err_diag.destroy()
        except:
            err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
            err_diag.run()
            err_diag.destroy()
        else:
            try:
                ver_list = winex_get.winex_versions()
            except socket.gaierror, e:
                err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to connect to server\n%s" % e[1] )
                err_diag.run()
                err_diag.destroy()
                return
            except:
                err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
                err_diag.run()
                err_diag.destroy()
                return
            if len( ver_list ):
                latest = ver_list[len( ver_list ) - 1]
                for ver in self.Point2Play.versions:
                    if ver["version"] == latest["version"]:
                        err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, "You already have the latest version. Overwrite?" )
                        resp = err_diag.run()
                        err_diag.destroy()
                        if resp == gtk.RESPONSE_NO:
                            return
                        else:
                            break
                file_diag = winex_get_gui.WinexGetDialog( self.Point2Play, ver_list, latest["version"] )
                file_diag.set_transient_for( self )
                file_diag.set_title( "Download WineX %s" % latest["version"] )
                filename = file_diag.run()
                gtk.input_remove( file_diag.input_handler )
                file_diag.destroy()
                if filename:
                    dirname = os.path.join( self.Point2Play.gamedir, ".winex_ver", "winex-%s" % latest["version"] )
                    if os.path.islink( dirname ) or os.path.isfile( dirname ):
                        os.unlink( dirname )
                    elif os.path.isdir( dirname ):
                        shutil.rmtree( dirname )
                    os.makedirs( dirname )
                    cwd = os.getcwd()
                    os.chdir( dirname )
                    os.system( "tar xzf %s" % filename )
                    os.chdir( cwd )
                    for i in range(0, len( self.Point2Play.versions ) ):
                        if self.Point2Play.versions[i]["version"] == latest["version"]:
                            del self.Point2Play.versions[i]
                            break
                    self.Point2Play.versions.append( { "version": latest["version"], "path": dirname } )
                    os.unlink( filename )
                    self.Point2Play.default_winex = latest["version"]
                    self.Point2Play.write()
                    self.widgets["menu_items"].get_item("/File/Install").set_sensitive( 1 )
                    self.widgets["install_button"].set_sensitive( 1 )
                    

    def update_config_widgets( self ):
        self.update_config = 1
        self.widgets["proxy_check"].set_active( self.config["proxy_check"] )
        self.widgets["proxy_host"].set_text( self.config["proxy_host"] )
        self.widgets["proxy_port"].set_text(str( self.config["proxy_port"] ) )
        self.widgets["username"].set_text( self.config["username"] )
        self.widgets["password"].set_text( self.config["password"] )
        # self.widgets["gamedir"].set_text( self.config["gamedir"] )
        for widget in ( "proxy_host", "proxy_port" ):
            self.widgets[widget].set_sensitive( self.config["proxy_check"] )
        self.update_config = 0
        if not len( self.config["username"] ):
            self.widgets["get_latest"].set_sensitive( 0 )

    def save_cb( self, widget, *args ):
        self.apply_cb( None, 0 )
        #print self.config["proxy_check"], self.Point2Play.config["proxy_check"]
        self.Point2Play.write()

    def play_cb( self, widget, *args ):
        # We want to play a game
        # First we get the name of the game
        program_sel = self.widgets["program_view"].get_selection().get_selected()
        if program_sel:
            program_model, program_iter = program_sel
            if program_iter:
                program_name = program_model.get_value( program_iter, PROGRAM_NAME_COL )
                game_name = program_model.get_value( program_iter, PROGRAM_GAME_COL )
                # We now have both..  let's do some magic!!  :-)
                for i in self.Point2Play._games[game_name]:
                    if i["Name"] == program_name:
                        try:
                            retval = self.Point2Play.winex( game_name, i["Path"], i["Workdir"] )
                        except Point2Play.Point2PlayError, e:
                            err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, e.__str__() )
                            err_diag.run()
                            err_diag.destroy()
                            return
                          
                        if not self.REAPING:
                            self.REAPING = 1
                            gtk.timeout_add( 1, self.reap_children )

    def reap_children( self, *args ):
        while 1:
            try:
                ret = os.waitpid( 0, os.WNOHANG )
                if ret[0] == 0:
                    gtk.timeout_add( 1, self.reap_children )
                    return
            except:
                self.REAPING = 0
                return

    def install_menu_cb( self, widget, *args ):
        install_dialog = install_gui.InstallProgramDialog( self )
        install_dialog.set_transient_for( self )
        ( install, game, dos_cwd, big_exe ) = install_dialog.run()
        install_dialog.destroy()
        while gtk.events_pending():
            gtk.main_iteration()
        if install and len( install ) > 0 and game and len( game ) > 0:
            new_game = 1
            for i in self.Point2Play.games():
                if game.lower() == i.lower():
                    game = i
                    new_game = 0
                    break
            if new_game:
                retval = self.Point2Play.new_game( game )
                if retval == 0:
                    self.widgets["game_view"].set_model( self.game_model( 1 ) )
                else:
                    if retval == 2:
                        err_msg = "Unable to run regapi"
                        err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, err_msg )
                        err_diag.run()
                        err_diag.destroy()
                        return
            retval = self.Point2Play.winex( game, install, None, self.Point2Play.gamedir + game + "/" + "games.ini.new", None, ( dos_cwd, big_exe )  )
            if not self.REAPING:
                self.REAPING = 1
                gtk.timeout_add( 1, self.reap_children )
            self.monitor_game_install( game )

    def rename_game_cb( self, widget, *args ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game = model.get_value( iter, 0 )
                diag = RenameGameDialog( self, game )
                resp = diag.run()
                diag.destroy()
                if resp == GAME_RENAMED:
                    self.widgets["game_view"].set_model( self.game_model( 1 ) )

    def remove_menu_cb( self, widget, *args ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game = model.get_value( iter, 0 )
            else:
                game = install_gui.RemoveGameDialog( self.Point2Play )
        else:
            game = install_gui.RemoveGameDialog( self.Point2Play )
        if game:
            # We confirm
            confirm_dialog = p2p_diag.P2PMessageDialog( self, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, "About to delete " + game + "\nAre you sure?" )
            response = confirm_dialog.run()
            confirm_dialog.destroy()
            if response == gtk.RESPONSE_YES:
                self.Point2Play.remove_game( game )
                self.widgets["game_view"].set_model( self.game_model( 1 ) )
                self.widgets["program_view"].set_model( gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING ) )
        if not len( self.Point2Play.games() ):
            self.widgets["remove_button"].set_sensitive( 0 )
            self.widgets["play_button"].set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/File/Play" ).set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/File/Remove" ).set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/File/Rename" ).set_sensitive( 0 )

    def mount_cb( self, *args ):
        if len( self.cdroms ) > 1:
            if self.CDDialog:
                self.CDDialog.show()
            else:
                self.CDDialog = mount.CDDialog( self.cdroms )
        else:
            if mount.drive_mounted( self.cdroms[0].getDev() ):
                (retval, message) = mount.mount_cd( "-u", self.cdroms[0].getDev() )
            else:
                (retval, message) = mount.mount_cd( "-m", self.cdroms[0].getDev() )
            if retval:
                diag = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CANCEL, message )
                diag.run()
                diag.destroy()

    def remove_icon_cb( self, selection, *args ):
        sel = self.widgets["program_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            game = model.get_value( iter, PROGRAM_GAME_COL )
            name = model.get_value( iter, PROGRAM_NAME_COL )
            yes_no_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, "Are you sure you want to delete %s?" % name )
            resp = yes_no_diag.run()
            yes_no_diag.destroy()
            if resp == gtk.RESPONSE_YES:
                try:
                    self.Point2Play.remove_program( game, name )
                except:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to remove " + name + " from " + game + " (%s)." % str( sys.exc_info()[1] ) )
                    err_diag.run()
                    err_diag.destroy()
                else:
                    self.Point2Play.write_programs( game )
                    self.widgets["program_view"].set_model( self.program_model(game) )

    def add_icon_cb( self, widget, *args ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            game = model.get_value( iter, 0 )
            exe_diag = install_gui.InstallExeDialog( self.Point2Play, game, self )
            add = exe_diag.run()
            exe_diag.destroy()
            if add:
                try:
                    self.Point2Play.new_program( game, add[0], add[1], add[2], add[3] )
                except:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to add %s to %s (%s)." % ( add[0], game, str( sys.exc_info()[1] ) ) )
                    err_diag.run()
                    err_diag.destroy()
                    return
                self.Point2Play.write_programs( game )
                self.widgets["game_view"].set_model( self.game_model( game ) )

    def kill_wine_cb( self, widget, *args ):
        proc_list = os.listdir("/proc")

        procs_to_kill = []
                                                                                
        for proc in proc_list:
            try:
                proc_num = int(proc)
                # It's a number, ie. a process
                try:
                    cmdline = file("/proc/" + proc + "/cmdline")
                    cmds = cmdline.read().split("\0")
                    if os.path.basename(cmds[0]) == "wine" or os.path.basename( cmds[0] ) == "wineserver":
                        procs_to_kill.append( proc_num )
                except:
                    pass
            except:
                pass

        for proc in procs_to_kill:
            try:
                os.kill( proc, signal.SIGTERM )
            except:
                pass
        gtk.timeout_add( 5, kill_wine_timeout, procs_to_kill )

    def about_menu_cb( self, widget, *args ):
        dialog = p2p_diag.P2PMessageDialog( self,
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    p2p_diag.MESSAGE_TG_LOGO,
                                    gtk.BUTTONS_OK,
                                    "TransGaming WineX 3.0 Point2Play\nVersion 1.0\n\nCopyright (C) 2003 by TransGaming Technologies Inc." )
        dialog.run()
        dialog.destroy()

    def contact_menu_cb( self, widget, *args ):
        dialog = p2p_diag.P2PMessageDialog( self,
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    p2p_diag.MESSAGE_TG_LOGO,
                                    gtk.BUTTONS_OK,
                                    "Contact: info@transgaming.com" )
        dialog.run()
        dialog.destroy()

    def support_menu_cb( self, widget, *args ):
        dialog = p2p_diag.P2PMessageDialog( self,
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    p2p_diag.MESSAGE_TG_LOGO,
                                    gtk.BUTTONS_OK,
                                    "Support: support@transgaming.com" )
        dialog.run()
        dialog.destroy()

    def game_select_cb( self, selection ):
        sel = selection.get_selected()
        if sel:
            sensitive = 0
            model, iter = sel
            if iter:
                game = model.get_value( iter, 0 )
                model = self.program_model( game )
                if game != "All":
                    sensitive = 1
            else:
                model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING )
            self.widgets["program_view"].set_model( model )
            self.widgets["play_button"].set_sensitive( 0 )
            self.widgets["menu_items"].get_item( "/Tools/Icons/Add" ).set_sensitive( sensitive )
            # self.widgets["menu_items"].get_item( "/Tools/Preferences" ).set_sensitive( sensitive )

    def program_select_cb( self, selection ):
        # self.widgets["meny_items"].get_widget( "/File/Play A Game" ).set_sensitive( 1 )
        sel = selection.get_selected()
        if sel:
            model, iter = sel
            if iter:
                if len( self.Point2Play.versions ):
                    play_sensitive = 1
                else:
                    play_sensitive = 0
                sensitive = 1
            else:
                play_sensitive = 0
                sensitive = 0
            for i in ( "configure_button", "remove_button" ):
                self.widgets[i].set_sensitive( sensitive )
            self.widgets["play_button"].set_sensitive( play_sensitive )
            self.widgets["menu_items"].get_item( "/Tools/Icons/Remove" ).set_sensitive( sensitive )
            self.widgets["menu_items"].get_item( "/File/Play" ).set_sensitive( play_sensitive )

    def configure_button_cb( self, button ):
        game = None
        program = None
        game_sel = self.widgets["game_view"].get_selection().get_selected()
        if game_sel:
            model, iter = game_sel
            if iter:
                game = model.get_value( iter, 0 )
                program_sel = self.widgets["program_view"].get_selection().get_selected()
                if program_sel:
                    model, iter = program_sel
                    if iter:
                        program = model.get_value( iter, 0 )
        dialog = config_gui.ConfigGameDialog( self.Point2Play, game, program )
        dialog.set_transient_for( self )
        dialog.run()
        dialog.destroy()

    def game_model( self, all = 0 ):
        model = gtk.ListStore( gobject.TYPE_STRING )
        if self.Point2Play._games:
            if len( self.Point2Play.games() ) > 1:
                if all:
                    iter = model.append()
                    model.set_value( iter, 0, "All" )
            for game in self.Point2Play.games():
                iter = model.append()
                model.set_value( iter, 0, game )
        return model

    def program_model( self, game, model = None ):
        """
        Function called to get a list model for the executables.

        Parameters:
            game  = game name
            model = Are we appending to an already existing model?
        """
        if not model: # Are we appending or creating a new list
            model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING )
        if self.Point2Play._games: # Do we have any games to list?
            if game == "All": # Special case of all executables in all games
                for game_list in self.Point2Play.games():
                    self.program_model( game_list, model )
            else:
                for program in self.Point2Play._games[game]: # List of programs
                    iter = model.append()             # Append iterator
                    model.set_value( iter, PROGRAM_NAME_COL, program["Name"] ) # Set game name
                    model.set_value( iter, PROGRAM_GAME_COL, game )
                    try:
                        pbuf = gtk.gdk.pixbuf_new_from_file( program["Icon"] )
                    except:
                        pbuf = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, 1, 8, ICON_WIDTH, ICON_HEIGHT )
                        pbuf.fill( 0 )
                    if pbuf.get_width() != ICON_WIDTH or pbuf.get_height() != ICON_HEIGHT:
                        pbuf = pbuf.scale_simple( ICON_WIDTH, ICON_HEIGHT, gtk.gdk.INTERP_HYPER )
                    model.set_value( iter, PROGRAM_ICON_COL, pbuf )
        return model

    def conf_panel_cb( self, selection ):
        sel = selection.get_selected()
        if sel:
            model, iter = sel

            if iter:
                print "Selected ", model.get_value( iter, 0 )
            else:
                print "Unselected"

    def prefs_menu_cb( self, widget, *args ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game = model.get_value( iter, 0 )        
                dialog = preferences_gui.PreferencesDialog( self.Point2Play, game )
                dialog.set_title( "Preferences %s" % game )
                dialog.run()
                dialog.destroy()

    def monitor_game_install( self, game ):
        if not self.monitor:
             self.monitor = [ game, ]
             gtk.timeout_add( 2000, self.monitor_cb )
        else:
            for i in self.monitor:
                if i == game:
                    return
            self.monitor.append( game )


    def monitor_cb( self ):
        if not self.monitor:
            return

        gtk.timeout_add( 2000, self.monitor_cb )

        for i in self.monitor:
            try:
                path = self.Point2Play.gamedir + i + "/" + "games.ini"
                os.rename( path + ".new",  path + ".tmp" )
                fp = file( path + ".tmp", "r" )
                lines = fp.readlines()
                os.unlink( path + ".tmp" )
                added_programs = 0
                while len( lines ) >= 4:
                    added_programs = 1
                    name = lines[0].strip()
                    del lines[0]
                    executable = lines[0].strip()
                    del lines[0]
                    icon = lines[0].strip()
                    del lines[0]
                    workdir = lines[0].strip()
                    del lines[0]
                    if name.lower().find( "uninstall" ) != -1:
                        print "Skipping uninstall icon: %s" % ( name )
                    else:
                        self.Point2Play.new_program( i, name, executable, icon, workdir )
                        added_programs = 1
                if added_programs:
                    self.Point2Play.write_programs( i )
                    # We must check to see if it's currently selected game

            except:
                pass
                # print "Found nothing for i this run"


    def tests_run_cb(self, widget, *args):
        test_sel = self.widgets["tests_testlist"].get_selection().get_selected()
        if test_sel:
            test_model, test_iter = test_sel
            if test_iter:
                test_name = test_model.get_value( test_iter, 0 )
                self.Point2Play.tests.runTest(test_name, self)

    def tests_selection_cb(self, widget, *args):
        self.test_area_labels_clear()
        test_sel = self.widgets["tests_testlist"].get_selection().get_selected()
        if test_sel:
            test_model, test_iter = test_sel
            if test_iter:
                test_name = test_model.get_value( test_iter, 0 )
                self.Point2Play.tests.setTestDescription(test_name, self)


    def test_area_labels_clear(self):
        while (len(self.widgets["test_area_label_boxs"]) > 0):
            box = self.widgets["test_area_label_boxs"][-1]
            self.widgets["test_area_table"].remove(box)
            self.widgets["test_area_labels"].remove(box.get_child())
            box.remove(box.get_child())
            del self.widgets["test_area_label_boxs"][-1]

        self.widgets["test_area_table"].resize(1, 2)

        if (len(self.widgets["test_area_label_boxs"]) > 0):
            raise Exception("internal error (boxs exists)")
        if (len(self.widgets["test_area_labels"]) > 0):
            raise Exception("internal error (labels exist)")

    def test_area_labels_add(self, colour_left, colour_right, text_left, text_right):
        numrows = self.widgets["test_area_table"].get_property("n-rows")
        if (numrows > 1 or (len(self.widgets["test_area_label_boxs"]) > 0 and len(self.widgets["test_area_labels"]) > 0)):
            self.widgets["test_area_table"].resize(numrows + 1, 2)
        else:
            numrows = numrows - 1

        self.widgets["test_area_labels"].append(gtk.Label(text_left))
        self.widgets["test_area_labels"][-1].set_padding(2, 5)
        self.widgets["test_area_labels"][-1].set_line_wrap(True)

        self.widgets["test_area_label_boxs"].append(gtk.EventBox())
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(colour_left))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_ACTIVE, gtk.gdk.color_parse(colour_left))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_INSENSITIVE, gtk.gdk.color_parse(colour_left))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse(colour_left))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse(colour_left))

        self.widgets["test_area_label_boxs"][-1].add(self.widgets["test_area_labels"][-1])

        self.widgets["test_area_labels"].append(gtk.Label(text_right))
        self.widgets["test_area_labels"][-1].set_padding(2, 5)
        self.widgets["test_area_labels"][-1].set_line_wrap(True)

        self.widgets["test_area_label_boxs"].append(gtk.EventBox())
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(colour_right))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_ACTIVE, gtk.gdk.color_parse(colour_right))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_INSENSITIVE, gtk.gdk.color_parse(colour_right))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse(colour_right))
        self.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse(colour_right))

        self.widgets["test_area_label_boxs"][-1].add(self.widgets["test_area_labels"][-1])

        self.widgets["test_area_table"].attach( self.widgets["test_area_label_boxs"][-2], 0, 1, numrows, \
             numrows+1, gtk.EXPAND | gtk.FILL, gtk.SHRINK | gtk.FILL, 0, 0)
        self.widgets["test_area_table"].attach( self.widgets["test_area_label_boxs"][-1], 1, 2, numrows, \
             numrows+1, gtk.SHRINK | gtk.FILL, gtk.SHRINK | gtk.FILL, 0, 0)

        self.widgets["test_area_label_boxs"][-1].show()
        self.widgets["test_area_label_boxs"][-2].show()

        self.widgets["test_area_labels"][-1].show()
        self.widgets["test_area_labels"][-2].show()

    def proxy_check_toggled( self, widget, *args ):
        self.config["proxy_check"] = widget.get_active()
        self.widgets["proxy_host"].set_sensitive( self.config["proxy_check"] )
        self.widgets["proxy_port"].set_sensitive( self.config["proxy_check"] )

class RenameGameDialog( gtk.Dialog ):
    def __init__(self, P2P_gui, game ):
        gtk.Dialog.__init__(self)
        self.set_transient_for( P2P_gui )
        self.orig_game = game
        self.set_title( "Rename " + game )
        self.P2P = P2P_gui.Point2Play
        vbox = self.get_children()[0]
        vbox.add( gtk.Label( "Rename " + game + " to " ) )
        self.text_entry = gtk.Entry()
        vbox.add( self.text_entry )
        button = self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        button.set_name( "round_button" )
        button = self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        button.set_name( "round_button" )
        vbox.show_all()

    def run( self ):
        while 1:
            resp = gtk.Dialog.run( self )
            if resp == gtk.RESPONSE_OK:
                try:
                    self.P2P.rename_game( self.orig_game, self.text_entry.get_text() )
                    return GAME_RENAMED
                except ( Point2Play.Point2PlayError, IOError ), e:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, str(e) )
                    err_diag.run()
                    err_diag.destroy()
            else:
                return GAME_NOT_RENAMED

if __name__ == '__main__':

    import license, sys

    skin = 1
    for i in sys.argv:
        if i == "-noskin":
            skin = 0

    try:
        P2PPATH=os.environ["P2PPATH"] + "/"
    except:
        P2PPATH=""

    if skin:
        gtk.rc_parse( P2PPATH+"Point2Play.rc")

    config_file = os.environ["HOME"] + "/.point2playrc"

    if not os.path.isfile( config_file ):
        shutil.copyfile( P2PPATH+"point2playrc", config_file )
    try:
        feg = Point2Play_gui( Point2Play.Point2Play( config_file ) )
    except license.licenseError, e:
        pass
    except:
        print str( sys.exc_info()[1] )
    else:
        feg.connect( 'destroy', gtk.mainquit )
        gtk.main()

