#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.
"""

import string
import os
import stat
import mount

#related exceptions
class cdromError(Exception):
    """base class for cdrom errors"""

class notCDorDVDFstabEntryError(cdromError):
    """thrown when the given fstab line is not a cdrom or dvd drive"""

    def __init__(self, fstabline):
        self.fstabline = fstabline

    def __str__(self):
        return `self.fstabline`

class fstabCdrom:
    """A cdrom device item, from the fstab"""

    def __init__(self, fstabline):
        if not self.passFstabLine(fstabline):
            raise notCDorDVDFstabEntryError(fstabline)

    def passFstabLine(self, line):
        """returns true if the line passed in as 'line'
        is probably a cd or dvd drive
        sets the member variables device_path, mount_path, mount_type and options_dev
        to their required parts (optoins_dev comes from the dev= option).

        this MUST give the exact same output as the winex cdrom test does
        wine/files/drive.c:DRIVE_ProbeCD"""
        # hence the ugly nature of this code
        items = line.split()
        if len( items ) == 0:
            return False
        # device
        if (items[0][0] != '/' and items[0][0:4] != 'none'):
            return False
        self.device_path = items[0]
        # mount point
        if (items[1][0] != '/'):
            return False
        self.mount_path = items[1]
        if (self.mount_path[-1] == '/'):
            self.mount_path = self.mount_path[:-1]
        # type
        self.mount_type = items[2]
        # mount options (dev= part)
        self.options_dev = ''
        dev_loc = items[3].find('dev=');
        if (dev_loc >= 0):
            end_dev_loc = items[3].find(',', dev_loc)
            self.options_dev = items[3][dev_loc:end_dev_loc]

        # now for the test
        if (str.find(self.mount_type, "iso9660") == -1 and
          str.find(self.mount_type, "udf") == -1 and
          self.mount_type != "auto" and
          self.mount_type != "supermount"):
            return False
        iscddvd = False
        if (self.device_path.find('cdr') >= 0): iscddvd = True
        elif (self.device_path.find('scd') >= 0): iscddvd = True
        elif (self.device_path.find('dvd') >= 0): iscddvd = True
        elif (self.mount_path.find('cdr') >= 0): iscddvd = True
        elif (self.mount_path.find('dvd') >= 0): iscddvd = True
        return iscddvd

    def getDev(self):
        if (self.device_path == 'none' and self.options_dev): return self.options_dev
        return self.device_path

    def isCDDeviceReadable(self):
        """checks if the current user has permissions to read the cd device"""
        stat_result = os.stat(self.getDev())
        gid = os.getgid()
        uid = os.getuid()
        if (stat_result[stat.ST_UID] == uid):
            if (stat_result[stat.ST_MODE] & stat.S_IRUSR):
                return True
        elif (stat_result[stat.ST_GID] == gid):
            if (stat_result[stat.ST_MODE] & stat.S_IRGRP):
                return True
        else:
            if (stat_result[stat.ST_MODE] & stat.S_IROTH):
                return True
        return False

    def isCDDeviceWriteable(self):
        """checks if the current user has permissions to write to the cd device"""
        stat_result = os.stat(self.getDev())
        gid = os.getgid()
        uid = os.getuid()
        if (stat_result[stat.ST_UID] == uid):
            if (stat_result[stat.ST_MODE] & stat.S_IWUSR):
                return True
        elif (stat_result[stat.ST_GID] == gid):
            if (stat_result[stat.ST_MODE] & stat.S_IWGRP):
                return True
        else:
            if (stat_result[stat.ST_MODE] & stat.S_IWOTH):
                return True
        return False

    def isCDPathMounted(self):
        if mount.drive_mounted(self.getDev()):
            return True
        return False

