"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""
import types

class config_file:
    def __init__( self, filename ):
        self._config_file__sections = []
        self._config_file__trailing_comments = []
        comment = []   # The list of 'current' comments
        options = []   # The list of 'current' options
        section = None # The 'current' section

        f = file( filename )  # Create file object from filename
        lines = f.readlines() # Get a list of lines
        """Check to see if first line is proper WINE REGISTRY --- this needs fixing"""
        if lines[0].rstrip().lower() != "wine registry version 2":
            print "Bad registry"
            return None

        del lines[0] # Remove the first line
        """
        Next we parse the rest of the config file lines... because we have
        to maintain the comments the parsing isn't as simple as could be

        Update:
        The file isn't a .INI style file..  it contains subsections which
        make a 'stateless' parser impossible.
        """
        for line in lines:
            line = line.strip() # Strip trailing newlines
            if len( line ) == 0:         # Empty line?
                continue
            if line[0] == ";" or line[0] == "#": # Are we a comment line?
                comment.append( line )
            elif line[0] == "[":                 # Are we a section header?
                section_name = line[1:len(line)-1]
                # print "Got new section path: %s" % ( section_name )
                node = None
                while 1:
                    index = section_name.find( "\\\\" )
                    if index == -1:
                        key = section_name
                        node = self.get_sub_section( node, key, 1 )
                        break
                    else:
                        key = section_name[:index]
                        section_name = section_name[index+2:]
                        node = self.get_sub_section( node, key, 1 )
                section = node
                options = section["options"]
                section["comments"].extend(comment)
                comment = []
            elif  line[0] == "\"":               # Are we an option?
                option_name, value = line.split( "=", 1 )
                option_name = option_name.strip()
                """Strip out the quotes in the names"""
                if option_name[0] == '"' and option_name[ len( option_name ) - 1 ] == '"':
                    option_name = option_name[ 1:len( option_name )-1 ]
                value = value.strip()
                """Strip out the quotes in the values"""
                if value[0] == '"' and value[ len ( value ) - 1 ] == '"':
                    value = value[ 1:len( value ) - 1 ]
                """Create an option dict with option name, value and the comments on top"""
                options.append( { "name": option_name, "value": value, "comments": comment } )
                comment = [] # Clear comments
            else:
                pass
              
        self._config_file__trailing_comments = comment
        # print self.sections()
        f.close()

    def write( self, filename ):
        f = file( filename, "w+" )
        # print self.sections()
        f.write( "WINE REGISTRY Version 2\n" )
        for section in self._config_file__sections:
            self.write_sub_section(f, section["name"], section)
        for comment in self._config_file__trailing_comments:
            f.write( comment + "\n" )
        # print self.sections()
        f.close()

    def write_sub_section( self, f, path, section ):
        for comment in section[ "comments" ]:
            f.write( comment + "\n" )
        if len(section[ "options" ]):
            f.write( "[" + path + "]\n" )
            for options in section[ "options" ]:
                for comment in options[ "comments" ]:
                    f.write( comment + "\n" )
                f.write( "\"" + options[ "name" ] + "\" = \"" + options[ "value" ] + "\"\n" )
            f.write( "\n" )
        for sub_sec in section[ "sections" ]:
            self.write_sub_section(f, path + "\\\\" + sub_sec["name"], sub_sec )

    def sections(self):
        sec_list = []
        for section in self._config_file__sections:
            sec_list.append( section[ "name" ] )
        return list( sec_list )

    def has_section( self, section ):
        for sec in self._config_file__sections:
            if sec[ "name" ] == section:
                return 1
        return 0

    def add_section( self, name, options, comments = () ):
        self._config_file__sections.append( { "name": name, "options": options, "comments": comments, "sections": [] } )

    def get_section( self, section, super_section = None):
        if super_section != None:
            appdefs = self.get_sub_section( None, "AppDefaults" )
            if appdefs == None:
                return None
            super_sec = self.get_sub_section( appdefs, super_section )
            if super_sec == None:
                return None
            sections = super_sec["sections"]
        else:
            sections = self._config_file__sections
        for sec in sections:
            if sec[ "name" ].lower() == section.lower():
                return sec
        return None

    def get_sub_section( self, root, section, make_it = 0 ):
        if root == None:
            sections = self._config_file__sections
        else:
            sections = root["sections"]
        for sec in sections:
            if sec[ "name" ].lower() == section.lower():
                return sec
        if make_it:
            sec = { "name": section, "comments": [], "options": [], "sections": [] }
            sections.append(sec)
            return sec
        return None

    def get_option( self, section, option, super_section = None, toplevel_ok = 0 ):
        if super_section != None:
            super_sec = self.get_section( section, super_section )
            if super_sec != None:
                for opt in super_sec[ "options" ]:
                    if opt[ "name" ] == option:
                        return opt[ "value" ]
            else:
                if toplevel_ok == 0:
                    return None
        sec = self.get_section( section )
        if sec != None:
            for opt in sec[ "options" ]:
                if opt["name"] == option:
                  return opt["value"] 
        else:
            return None

    def set_option( self, section, option, value, super_section = None, toplevel_ok = 0 ):
        # print "[%s]\n%s=%s\n" % ( section, option, value )
        if super_section != None:
            appdefs = self.get_sub_section( None, "AppDefaults", 1 )
            super_sec = self.get_sub_section( appdefs, super_section, 1 )
            sec = self.get_sub_section( super_sec, section, 1 )
            for opt in sec["options"]:
                if opt["name"] == option:
                    sec["options"][sec["options"].index(opt)] = { "name": option,
                                 "value": value,
                                 "comments": opt["comments"] }
                    return
            sec["options"].append( { "name": option,
                                     "value": value,
                                     "comments": [] } )
        else:
            sec = self.get_section( section )
            if sec == None:
                   sec = { "name": section, "comments": [], "options": [ { "name": option, "value": value, "comments": [] } ], "sections": [] }
                   self._config_file__sections.insert( 1, sec )
            else:
                for opt in sec["options"]:
                    if opt["name"] == option:
                        sec["options"][sec["options"].index(opt)] = { "name": option, "value": value, "comments": opt["comments"] }

    def drive_index( self, drive ):
        i = 0
        for sec in self._config_file__sections:
            if sec["name"].lower() == "drive " + drive.lower():
                return i
            i = i + 1
        return -1

    def get_drive( self, drive ):
        index = self.drive_index( drive )
        if index >= 0:
            return self._config_file__sections[index]["options"]

    def set_drive( self, drive, path, type, label, filesystem ):
        section = { "name"   : "Drive " + drive.upper(),
                    "options": ( { "name"     : "Path",
                                   "value"    : path,
                                   "comments" : () },
                                 { "name"     : "Type",
                                   "value"    : type,
                                   "comments" : () },
                                 { "name"     : "Label",
                                   "value"    : label,
                                   "comments" : () },
                                 { "name"     : "Filesystem",
                                   "value"    : filesystem,
                                   "comments" : () } ),
                    "comments": [],
                    "sections": [] }
        index = self.drive_index( drive )
        if self.drive_index( drive ) >= 0:
            self._config_file__sections[index] = section
        else:
            self._config_file__sections.append( section )

