"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

try:
    import gtk, gobject
except:
    raise config_guiError( "Unable to load GTK2 Python bindings" )

import types, config, copy, os.path

class config_guiError( Exception ):
    def __init__( self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

CHECK  = 1
COMBO  = 2
STRING = 3

desktops = ( "No", "640x480", "800x600", "1024x768", "1280x1024", "1600x1200" )

winvers  = ( "win95", "win98", "winme", "nt351", "nt40", "win2k", "winxp" )

game_options = ( { "name"   : "Managed",
                   "type"   : CHECK,
                   "default": 'Y',
                   "section": "x11drv",
                   "Label"  : "Managed",
                   "Tips"   : "Have the window manager manage the window",
                   "X-Tips" : "" },
                 { "name"   : "Desktop",
                   "type"   : COMBO,
                   "default": 'No',
                   "section": "x11drv",
                   "Label"  : "Desktop",
                   "List"   : desktops,
                   "Tips"   : "Run inside a desktop window",
                   "X-Tips" : "" },
                 { "name"   : "UseXVidMode",
                   "type"   : CHECK,
                   "default": 'Y',
                   "section": "x11drv",
                   "Label"  : "Use Xvid Mode",
                   "Tips"   : "Use XF86VidMode to change resolutions",
                   "X-Tips" : "" },
                 { "name"   : "DXGrab",
                   "type"   : CHECK,
                   "default": 'Y',
                   "section": "x11drv",
                   "Label"  : "DXGrab",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "UseMMap",
                   "type"   : CHECK,
                   "default": 'N',
                   "section": "wineoss",
                   "Label"  : "UseMMap",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "FullDuplex",
                   "type"   : CHECK,
                   "default": 'N',
                   "section": "wineoss",
                   "Label"  : "Full Duplex",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "windows",
                   "type"   : COMBO,
                   "default": 'win98',
                   "section": "version",
                   "Label"  : "Winver",
                   "List"   : winvers,
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "debugmsg",
                   "type"   : STRING,
                   "default": '',
                   "section": "transgaming",
                   "Label"  : "Debug",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "cmdline",
                   "type"   : STRING,
                   "default": '',
                   "section": "transgaming",
                   "Label"  : "Command line",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "AnisotropicTextureFiltering",
                   "type"   : CHECK,
                   "default": "N",
                   "section": "d3dgl",
                   "Label"  : "Anisotropic Filtering",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "VertexShaders",
                   "type"   : CHECK,
                   "default": "Y",
                   "section": "d3dgl",
                   "Label"  : "Vertex Shaders",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "ClipSpaceFix",
                   "type"   : CHECK,
                   "default": "Y",
                   "section": "d3dgl",
                   "Label"  : "Clip Space Fix",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "bigexe",
                   "type"   : CHECK,
                   "section": "transgaming",
                   "default": "N",
                   "Label"  : "Big EXE",
                   "Tips"   : "",
                   "X-Tips" : "" },
                 { "name"   : "doscwd",
                   "type"   : STRING,
                   "section": "transgaming",
                   "default": "",
                   "Label"  : "Working Directory",
                   "Tips"   : "",
                   "X-Tips" : "" } )

def _get_options( game_config, executable, options ):
    _options = None
    for i in options:
        val = game_config.get_option( i["section"], i["name"], executable, 1 )
        if not val:
            val = i["default"]
        if _options:
            _options[i["name"]] = val
        else:
            _options = { i["name"]: val }
    return _options

class ConfigGameDialog(gtk.Dialog):
    def __init__( self, Point2Play, game, executable ):
        self._games = None
        self.Point2Play = Point2Play
        gtk.Dialog.__init__( self ) # Init base class
        vbox = self.get_children()[0]
        """Build left side tree"""
        model = gtk.TreeStore( gobject.TYPE_STRING, gobject.TYPE_STRING )
        tree = gtk.TreeView( model )
        tree.get_selection().set_mode('single')
        tree.get_selection().connect( 'changed', self.game_select_cb )
        for game_name in Point2Play.games():
            game_config = config.config_file( Point2Play.gamedir + game_name + "/config" )
            if not self._games:
                self._games = { game_name: { "__main__": _get_options( game_config, None, game_options ) } }
            else:
                self._games[game_name] = { "__main__": _get_options( game_config, None, game_options ) }
            top_iter = model.append( None )
            model.set_value( top_iter, 0, game_name )
            model.set_value( top_iter, 1, "__main__" )
            if game == game_name and not executable:
                tree.get_selection().select_iter( top_iter )
            for exe_name in Point2Play._games[game_name]:
                real_exe = os.path.basename( exe_name["Path"] )
                self._games[game_name][exe_name["Name"]] = _get_options( game_config, real_exe, game_options )
                iter = model.append( top_iter )
                model.set_value( iter, 0, exe_name["Name"] )
                model.set_value( iter, 1, game_name )
                if game == game_name and executable == exe_name["Name"]:
                    tree.get_selection().select_iter( iter )
        self._orig_games = copy.deepcopy( self._games )
        cell = gtk.CellRendererText()
        col = gtk.TreeViewColumn( "Program", cell, text=0 )
        tree.append_column( col )
        sw = gtk.ScrolledWindow()
        sw.set_size_request( 250, 300 )
        sw.add( tree )
        hbox = gtk.HBox()
        hbox.pack_start( sw, gtk.FALSE, gtk.FALSE )
        table = gtk.Table( 2, len( game_options ) )
 
        i = 0
        self.widgets = { "table": table, "game_view": tree }
        config_tips = gtk.Tooltips()
        for option in game_options:
            label = gtk.Label( option["Label"] )
            if option["Tips"] and len( option["Tips"] ):
                config_tips.set_tip( label, option["Tips"], option["X-Tips"] )
            table.attach( label, 0, 1, i, i + 1 )
            if option["type"] == CHECK:
                widget = gtk.CheckButton()
                widget.connect( 'toggled', self.checkbutton_cb, option["name"] )
            elif option["type"] == COMBO:
                widget = gtk.Combo()
                widget.set_popdown_strings( option["List"] )
                widget.entry.set_editable( 0 )
                widget.entry.connect( 'changed', self.textentry_cb, option["name"] )
            elif option["type"] == STRING:
                widget = gtk.Entry()
                widget.connect( 'changed', self.textentry_cb, option["name"] )
            else:
                print "Unknown object type"
            self.widgets[option["name"]] = widget
            table.attach( widget, 1, 2, i, i + 1 )
            i = i + 1
        temp_vbox = gtk.VBox()
        temp_vbox.pack_start( table, gtk.FALSE, gtk.FALSE )
        hbox.pack_start( temp_vbox, gtk.FALSE, gtk.FALSE )
        vbox.add( hbox )
        hbox.show_all()
        hbox = gtk.HBox()
        self.widgets["restore_button"] = gtk.Button( "Restore" )
        self.widgets["restore_button"].set_name( "round_button" )
        self.widgets["apply_button"] = gtk.Button( "Apply" )
        self.widgets["apply_button"].set_name( "round_button" )
        self.widgets["defaults_button"] = gtk.Button( "Defaults" )
        self.widgets["defaults_button"].set_name( "round_button" )
        self.widgets["exit_button"] = gtk.Button( "Exit" )
        self.widgets["exit_button"].set_name( "round_button" )

        self.widgets["restore_button"].connect( 'clicked', self.restore_cb )
        self.widgets["apply_button"].connect( 'clicked', self.apply_cb )
        self.widgets["defaults_button"].connect( 'clicked', self.defaults_cb )
        self.widgets["exit_button"].connect( 'clicked', self.exit_cb )

        hbox.add( self.widgets["restore_button"] )
        hbox.add( self.widgets["apply_button"] )
        hbox.add( self.widgets["defaults_button"] )
        hbox.add( self.widgets["exit_button"] )

        hbox.show_all()
        vbox.pack_start( hbox, gtk.FALSE, gtk.FALSE )
        self.updating = 0
        self.set_title("Configure")
        config_tips.enable()
        self.set_size_request( 600, 400 )
        self.set_resizable( 0 )

    def checkbutton_cb( self, checkbutton, name ):
        if self.updating:
            return
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                if checkbutton.get_active():
                    self._games[game_name][exe_name][name] = 'Y'
                else:
                    self._games[game_name][exe_name][name] = 'N'
                # print "Updating checkbutton ", name
                self.widgets["apply_button"].set_sensitive( 1 )
                self.widgets["restore_button"].set_sensitive( 1 )

    def textentry_cb( self, textentry, name ):
        if self.updating:
            return
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                # print "Updating..."
                self._games[game_name][exe_name][name] = textentry.get_text()
                self.widgets["apply_button"].set_sensitive( 1 )
                self.widgets["restore_button"].set_sensitive( 1 )

    def game_select_cb( self, selection ):
        self.update_config_widgets()
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                self.set_title( "Configure " + model.get_value( iter, 0 ) )
                return
        self.set_title( "Configure" )

    def restore_cb( self, button ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                self._games[game_name][exe_name] = copy.copy( self._orig_games[game_name][exe_name] )
                self.update_config_widgets()

    def apply_cb( self, button ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                self._orig_games[game_name][exe_name] = copy.deepcopy( self._games[game_name][exe_name] )
                game_config = config.config_file( self.Point2Play.gamedir + game_name +  "/config" )
                if exe_name == "__main__":
                    real_exe = None
                else:
                    for exe in self.Point2Play._games[game_name]:
                        if exe["Name"] == exe_name:
                            real_exe = os.path.basename( exe["Path"] )
                            break
                for option in game_options:
                    game_config.set_option( option["section"], option["name"], self._games[game_name][exe_name][option["name"]], real_exe )
                game_config.write( self.Point2Play.gamedir + game_name + "/config" )

    def defaults_cb( self, button ):
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                for option in game_options:
                    self._games[game_name][exe_name][option["name"]] = option["default"]
                self.update_config_widgets()

    def exit_cb( self, button ):
        self.response( 0 )

    def update_config_widgets( self ):
        self.updating = 1
        widgets_changed = 0
        sel = self.widgets["game_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game_name = model.get_value( iter, 0 )
                exe_name = model.get_value( iter, 1 )
                if exe_name != "__main__":
                    self.widgets["table"].show()
                    temp = game_name
                    game_name = exe_name
                    exe_name = temp
                    for i in ( "defaults", "exit" ):
                        self.widgets[i+"_button"].set_sensitive( 1 )
                else:
                    self.widgets["table"].hide()
                    for i in ("restore", "apply", "defaults", "exit" ):
                        self.widgets[i+"_button"].set_sensitive( 0 )
                    return 
                self.updating = 1
                widgets_changed = 0
                for option in game_options:
                    if not self._games[game_name][exe_name][option["name"]] == self._orig_games[game_name][exe_name][option["name"]]:
                        widgets_changed = 1
                    if option["type"] == CHECK:
                        if self._games[game_name][exe_name][option["name"]] == 'Y':
                            active = 1
                        else:
                            active = 0
                        self.widgets[option["name"]].set_active( active )
                    elif option["type"] == COMBO:
                        self.widgets[option["name"]].entry.set_text( self._games[game_name][exe_name][option["name"]] )
                    elif option["type"] == STRING:
                        self.widgets[option["name"]].set_text( self._games[game_name][exe_name][option["name"]] )
        self.updating = 0
        if not widgets_changed:
            self.widgets["apply_button"].set_sensitive( 0 )
            self.widgets["restore_button"].set_sensitive( 0 )
        else:
            self.widgets["apply_button"].set_sensitive( 1 )
            self.widgets["restore_button"].set_sensitive( 1 )

class global_prefs_dialog( gtk.Dialog ):
    def __init__( self, Point2Play, game ):
        gtk.Dialog.__init__( self )
        self.set_title( "Global Preferences - " + game )
        model = gtk.TreeStore( gobject.TYPE_STRING )
        advanced_iter = model.append( None )
        model.set_value( advanced_iter, 0, "Advanced Options" )
        for i in ( "Configure Drives", "Configure Ports", "Configure Fonts" ):
            iter = model.append( advanced_iter )
            model.set_value( iter, 0, i )
        tree = gtk.TreeView( model )
        tree.get_selection().set_mode('single')
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( "Option", cell, text=0 )
        tree.append_column( column )
        drive_frame = gtk.Frame( "Drive Configuration" )
        ports_frame = gtk.Frame( "Port Configuration" )
        fonts_frame = gtk.Frame( "Font Configuration" )
        self.widgets = { "tree": tree }
        self.widgets["drive_frame"] = drive_frame
        self.widgets["ports_frame"] = ports_frame
        self.widgets["fonts_frame"] = fonts_frame
        hbox = gtk.HBox()
        self.widgets["hbox"] = hbox
        hbox.pack_start( tree, gtk.FALSE, gtk.FALSE )
        tree.get_selection().connect( 'changed', self.option_select )
        tree.set_size_request( 300, 300 )
        hbox.add( drive_frame )
        self.get_children()[0].add( hbox )
        self.get_children()[0].show_all()

    def option_select( self, selection ):
        sel = selection.get_selected()
        if sel:
            model, iter = sel
            if iter:
                option = model.get_value( iter, 0 )
                if option == "Configure Drives":
                    frame = self.widgets["drive_frame"]
                elif option == "Configure Ports":
                    frame = self.widgets["ports_frame"]
                elif option == "Configure Fonts":
                    frame = self.widgets["fonts_frame"]
                else:
                    frame = self.widgets["empty_frame"]
            else:
                frame = self.widgets["empty_frame"]
            child = self.widgets["hbox"].get_children()
            if len( child ) == 2:
                child[1].hide()
                self.widgets["hbox"].remove( child[1] )
            self.widgets["hbox"].add( frame )
            frame.show()
