"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

import gtk, os, p2p_diag

INSTALL_INSTALL = 1
INSTALL_CANCEL  = 2
INSTALL_MOUNT = 3

class InstallProgramDialog( gtk.Dialog ):
    def __init__( self, P2P_gui ):
        gtk.Dialog.__init__( self )
        self.set_title( "Install a Program" )
        vbox = self.get_children()[0]
        table = gtk.Table( 2, 5 )
        self.widgets = { "filename_entry": gtk.Entry() }
        button = gtk.Button( "Browse" )
        button.set_name( "round_button" )
        button.connect( 'clicked', self.filename_browse_cb )
        self.widgets["combo"] = gtk.Combo()
        if P2P_gui.Point2Play._games:
            self.widgets["combo"].set_popdown_strings( P2P_gui.Point2Play.games() )
        self.P2P_gui = P2P_gui
        tbutton = self.add_button( "(Un)mount", INSTALL_MOUNT )
        tbutton.set_name( "round_button" )
        tbutton = self.add_button( gtk.STOCK_CANCEL, INSTALL_CANCEL )
        tbutton.set_name( "round_button" )
        self.widgets["continue_button"] = self.add_button( "Continue", INSTALL_INSTALL )
        self.widgets["continue_button"].set_name( "round_button" )
        self.widgets["filename_entry"].connect( 'changed', self.sensitive_cb, self.widgets["continue_button"] )
        self.widgets["continue_button"].set_sensitive( 0 )

        table.attach( gtk.Label( "Path to Installer Executable" ), 0, 1, 0, 1 )
        table.attach( self.widgets["filename_entry"], 0, 1, 1, 2 )
        table.attach( button, 1, 2, 1, 2 )
        table.attach( gtk.Label( "Program Title" ), 0, 1, 2, 3 )
        table.attach( self.widgets["combo"], 0, 1, 3, 4 )
        self.widgets["dos_cwd"] = gtk.CheckButton( "Run Directory" )
        self.widgets["big_exe"] = gtk.CheckButton( "Oversized Exe File" )
        table.attach( self.widgets["big_exe"], 0, 1, 4, 5 )
        table.attach( self.widgets["dos_cwd"], 1, 2, 4, 5 )

        vbox.add( table )
        vbox.show_all()

    def run( self ):
        while 1:
            response = gtk.Dialog.run( self )

            if response == INSTALL_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
                return ( None, None, 0, 0 )
            elif response == INSTALL_MOUNT:
                self.P2P_gui.mount_cb( None )
                if len( self.P2P_gui.cdroms ) > 1:
                    self.P2P_gui.CDDialog.run()
            else:
                fname = self.widgets["filename_entry"].get_text()
                if( os.access( fname, os.F_OK ) ):
                    return ( fname, self.widgets["combo"].entry.get_text(), self.widgets["dos_cwd"].get_active(), self.widgets["big_exe"].get_active() )
                else:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CANCEL, fname + " does not exist" )
                    err_diag.run()
                    err_diag.destroy()

    def filename_browse_cb( self, button ):
        file_sel = p2p_diag.P2PFileSelection()
        filename = self.widgets["filename_entry"].get_text()
        if not len( filename ):
            filename = os.environ["HOME"] + "/"
            for cdrom in self.P2P_gui.cdroms:
                if (cdrom.isCDPathMounted()):
                    filename = cdrom.mount_path + "/"
        file_sel.set_filename( filename )
        response = file_sel.run()
        if response == gtk.RESPONSE_OK:
            self.widgets["filename_entry"].set_text( file_sel.get_filename() )
        file_sel.destroy()

    def sensitive_cb( self, widget, *widgets ):
        sensitive = len( widget.get_text() ) > 0
        for button in widgets:
            button.set_sensitive( sensitive )

def RemoveGameDialog( Point2Play, parent = None  ):
    dialog = gtk.Dialog( "Remove which games?" )
    vbox = dialog.get_children()[0]
    vbox.add( gtk.Label( "Remove which game?" ) )
    combo = gtk.Combo()
    combo.set_popdown_strings( Point2Play.games() )
    combo.entry.set_editable( 0 )
    vbox.add( combo )
    vbox.show_all()
    button = dialog.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
    button.set_name( "round_button" )
    button = dialog.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
    button.set_name( "round_button" )
    response = dialog.run()
    text = combo.entry.get_text()
    dialog.destroy()
    if response == gtk.RESPONSE_OK:
        return text
    else:
        return None

class InstallExeDialog( gtk.Dialog ):
    def __init__( self, Point2Play, game, parent = None ):
        gtk.Dialog.__init__( self )
        self.set_title( "Install program in %s" % game )
        self.game = game
        self.P2P = Point2Play
        if parent:
            self.set_transient_for( parent )
        vbox = self.get_children()[0]
        self.widgets = { "blah": 0 }
        dialog_items = ( ( "Name", "name", 0 ),
                         ( "Program", "program_te", 1 ),
                         ( "Icon", "icon_te", 1 ),
                         ( "Workdir", "workdir_te", 1 ) )
        table = gtk.Table( 3, len( dialog_items ) )
        i = 0
        for item in dialog_items:
            table.attach( gtk.Label( item[0] ), 0, 1, i, i+1 )
            self.widgets[ item[1] ] = gtk.Entry()
            table.attach( self.widgets[ item[1] ], 1, 2, i, i+1 )
            if item[2]:
                button = gtk.Button( "Browse" )
                button.set_name( "round_button" )
                table.attach( button, 2, 3, i, i+1 )
                button.connect( 'clicked', self.browse_cb, self.widgets[ item[1] ] )
            i=i+1
        vbox.add( table )
        vbox.show_all()
        button = self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        button.set_name( "round_button" )
        button = self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        button.set_name( "round_button" )

    def browse_cb( self, widget, text_entry ):
        file_diag = p2p_diag.P2PFileSelection()
        filename = text_entry.get_text()
        if not len( filename ):
            filename = os.path.join( self.P2P.gamedir, self.game, "c_drive" ) + "/"
        file_diag.set_filename( filename )
        resp = file_diag.run()
        filename = file_diag.get_filename()
        if resp == gtk.RESPONSE_OK:
            text_entry.set_text( filename )
        file_diag.destroy()
             
    def run( self ):
        while 1:
            resp = gtk.Dialog.run( self )
            if resp == gtk.RESPONSE_CANCEL or resp == gtk.RESPONSE_DELETE_EVENT:
                return None
            else:
                return ( self.widgets["name"].get_text(), self.widgets["program_te"].get_text(), self.widgets["icon_te"].get_text(), self.widgets["workdir_te"].get_text() )
    
