#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import glxtg

TESTS = ("Test For Hardware 3D Graphics Acceleration", "tgtest_gl", True),

def tgtest_gl(window):
    """run the glxtg test suite (from the .so file), that returns the FPS that glxgears
    can give us, so this module will then make an analysis based on that as to if the user has
    hardware acceleration"""

    result = glxtg.gearsfps(window)
    resultfps = result['fps']
    if (resultfps > 800):
        fps_text = "OK"
        fps_colour = "green"
    else:
        fps_text = "Slow"
        fps_colour = "red"
    if ((result['isdirect']) and (result['glmajor'] >= 1) and (result['glminor'] >= 2)):
        req_text = 'OK'
        req_colour = "green"
    elif (result['isdirect']):
        req_text = 'Incorrect OpenGL version\n%i.%i (requires 1.2)' % (result['glmajor'], result['glminor'])
        req_colour = "red"
    else:
        req_text = 'Not Available'
        req_colour = "red"

    res = (('OpenGL Direct Rendering', req_text, req_colour),
           ('3D Rendering Speed', fps_text, fps_colour),)
    return res

def tgtest_gl_description():
    return 'Performs several tests for hardware\nacceleration. This test will attempt to\ndetermine ' + \
           'if your graphics card is\nsuitable for running 3D-intense games\nwith WineX, and whether ' + \
           'it has been\nset up correctly.  The test will also run a\nbenchmark on your card ' + \
           'to try and gauge\nthe performance - during this test you\nshould see 3 rotating gears.\n\n' + \
           'This test may take several minutes\nto perform depending on your hardware.\n\n' + \
           'If this test fails, consult your\nLinux distribution documentation.'

if (__name__ == "__main__"):
    for test in TESTS:
        print "running test ", test[0]
        eval(test[1] + '()')

