"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

import gtk, gobject, winex_get, winex_get_gui, os, stat, sys, shutil, license, p2p_diag

VERSION_COL=0
PATH_COL=3

class WinexInstallDialog( gtk.Dialog ):
    def __init__(self, P2P, ver ):
        gtk.Dialog.__init__(self)
        self.P2P = P2P
        self.set_title( "Add/Remove WineX" )
        self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        vbox = self.get_children()[0]
        table = gtk.Table( 2, 2 )
        self.widgets = { "directory_entry": gtk.Entry() }
        button = gtk.Button( "Browse" )
        button.connect( 'clicked', self.dirname_browse_cb )
        table.attach( gtk.Label( "Directory" ), 0, 1, 0, 1 )
        table.attach( self.widgets["directory_entry"], 0, 1, 1, 2 )
        table.attach( button, 1, 2, 1, 2 )
        vbox.add( table )
        self.show_all()

    def dirname_browse_cb( self, *args ):
        dir_sel = p2p_diag.P2PFileSelection()
        filename = self.widgets["directory_entry"].get_text()
        if not len( filename ):
            filename = self.P2P.gamedir + "/"
        dir_sel.set_filename( filename )
        response = dir_sel.run()
        if response == gtk.RESPONSE_OK:
            self.widgets["directory_entry"].set_text( dir_sel.get_filename() )
        dir_sel.destroy()

    def run(self):
        while 1:
            response = gtk.Dialog.run(self)
            if response == gtk.RESPONSE_DELETE_EVENT:
                response = gtk.RESPONSE_CANCEL

            if response == gtk.RESPONSE_CANCEL:
                return None
            else:
                dirname = self.widgets["directory_entry"].get_text()
                if os.access( dirname, os.F_OK ): # It exists
                    if not os.path.isdir( dirname ):
                        err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CANCEL, dirname + " is a file." )
                        err_diag.run()
                        err_diag.destroy()
                    else:
                        # No file there, directory might already exist in P2P
                        in_ver_list = 0
                        for ver in self.P2P.versions:
                            if ver["path"] == dirname:
                                yes_no_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, "Version " + ver["version"] + " is already installed there.  Upgrade?" )
                                resp = yes_no_diag.run()
                                yes_no_diag.destroy()
                                if resp == gtk.RESPONSE_YES:
                                    return dirname
                                else:
                                    in_ver_list = 1
                                    break
                        if not in_ver_list:
                            yes_no_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, "Installing in this directory will destroy all contents currently in this dir.  Continue?" )
                            resp = yes_no_diag.run()
                            yes_no_diag.destroy()
                            if resp == gtk.RESPONSE_YES:
                                return dirname
                else:
                    yes_no_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, dirname + " does not exist.  Create?" )
                    resp = yes_no_diag.run()
                    yes_no_diag.destroy()
                    if resp == gtk.RESPONSE_YES:
                        try:
                            os.makedirs( dirname )
                            return dirname
                        except:
                            err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to create directory.\n" )
                            err_diag.run()
                            err_diag.destroy()

class WinexDialog(gtk.Dialog):
    def __init__(self,P2P):
        gtk.Dialog.__init__(self)
        self.P2P = P2P
        self.total_vers = winex_get.winex_versions( P2P )
        # We build the list here
        model = self.version_model()
        vbox = self.get_children()[0]
        hbox = gtk.HBox()

        list = gtk.TreeView( model )
        i = 0
        for label in [ "Version", "Installed", "Default", "Path" ]:
            cell = gtk.CellRendererText()
            column = gtk.TreeViewColumn( label, cell, text=i)
            column.set_alignment( 0.5 )
            list.append_column( column )
            i = i + 1
        list.get_selection().set_mode('single')
        list.get_selection().connect('changed', self.select_cb )
        vbox.add( hbox )
        sw = gtk.ScrolledWindow()
        sw.set_size_request( 350,200)
        sw.add( list )
        hbox.add( sw )
        vbox_button = gtk.VBox()
        self.widgets = { "version_list": list }
        for i in ( ( "install", "Install" ), ( "remove", "Remove" ), ( "default", "Set Default" ) ):
            self.widgets[i[0]] = gtk.Button( i[1] )
            vbox_button.pack_start( self.widgets[i[0]], gtk.FALSE, gtk.FALSE )
            self.widgets[i[0]].set_sensitive( 0 )
        self.widgets["install"].connect( 'clicked', self.install_cb )
        self.widgets["remove"].connect( 'clicked', self.remove_cb )
        self.widgets["default"].connect( 'clicked', self.default_cb )
        hbox.add( vbox_button )
        self.show_all()
        self.add_button( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE )

    def version_model( self ):
        model = gtk.ListStore( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
        for ver in self.total_vers:
            installed = "N"
            path = ""
            default = "N"
            for i in self.P2P.versions:
                if i["version"] == ver["version"]:
                    installed = "Y"
                    path = i["path"]
                    break
            if ver["version"] == self.P2P.default_winex:
                default = "Y"
            iter = model.append()
            model.set_value( iter, VERSION_COL, ver["version"] )
            model.set_value( iter, 1, installed )
            model.set_value( iter, 2, default )
            model.set_value( iter, PATH_COL, path )
        return model

    def install_cb( self, *args ):
        sel = self.widgets["version_list"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                ver = model.get_value( iter, VERSION_COL )
                install_diag = WinexInstallDialog( self.P2P, ver )
                dirname = install_diag.run()
                install_diag.destroy()
                if dirname and len( dirname ):
                    lic_fp = winex_get.winex_get_license( self.P2P, self.total_vers, ver )
                    if lic_fp:
                        lic_diag = license.LicenseDialog( lic_fp )
                        resp = lic_diag.run()
                        lic_diag.destroy()
                        if resp == license.LICENSE_DECLINE:
                            return None
             
                    file_diag = winex_get_gui.WinexGetDialog( self.P2P, self.total_vers, ver )
                    filename = file_diag.run()
                    gtk.input_remove( file_diag.input_handler )
                    file_diag.destroy()
                    if filename:
                        # We check if we are upgrading an old version of WineX
                        old_version = None
                        for version in self.P2P.versions:
                            if version["path"] == dirname:
                                try:
                                    os.rename( dirname, dirname + str( os.getpid() ) )
                                    old_version = version["version"]
                                except:
                                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to backup previous version (%s)." %  str( sys.exc_info()[1] ) )
                                    err_diag.run()
                                    err_diag.destroy()
                                    os.unlink( filename )
                                try:
                                    os.makedirs( dirname )
                                except:
                                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to create new directory %s (%s)." % ( dirname, str( sys.exc_info()[1] ) ) )
                                    err_diag.run()
                                    err_diag.destroy()
                                    os.unlink( filename )
                                    os.rename( dirname + str( os.getpid() ), dirname )
                                    return
                        
                        files = os.listdir( dirname )
                        if len( files ):
                            try:
                                for i in files:
                                    if os.path.isdir( os.path.join( dirname, i ) ):
                                         shutil.rmtree( os.path.join( dirname, i ) )
                                    else:
                                         os.unlink( os.path.join( dirname, i ) )
                            except:
                                err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Unable to delete contents of %s (%s)" % ( dirname, str( sys.exc_info()[1] ) ) )
                                err_diag.run()
                                err_diag.destroy()
                                os.unlink( filename )
                                return
                        cwd = os.getcwd()
                        os.chdir( dirname )
                        fp = file( filename, "rb" )
                        fw = os.popen( "tar xzf -", "w" )
                        data = fp.read( 4096 )
                        while len( data ) > 0:
                            fw.write( data )
                            data = fp.read( 4096 )
                        fp.close()
                        fw.close()
                        os.chdir( cwd )
                        self.P2P.versions.append( { "version": ver, "path": dirname } )
                        os.unlink( filename )
                        if old_version:
                            for i in range(0,len(self.P2P.versions)):
                                if self.P2P.versions[i]["version"] == old_version:
                                    del self.P2P.versions[i]
                                    break
                            shutil.rmtree( dirname + str( os.getpid() ) )
                        self.widgets["version_list"].set_model( self.version_model() )
                        self.P2P.write()

    def remove_cb( self, *args ):
        sel = self.widgets["version_list"].get_selection().get_selected()
        if sel:
            model, iter = sel
            version = model.get_value( iter, VERSION_COL )
            path = model.get_value( iter, PATH_COL )
            yes_no_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, "Are you sure you want to delete %s from %s?" % ( version, path ) );
            resp = yes_no_diag.run()
            yes_no_diag.destroy()
            if resp == gtk.RESPONSE_YES:
                try:
                    shutil.rmtree( path )
                except:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Error deleting %s from %s (%s)." % ( version, path, str( sys.exc_info()[1] ) ) )
                    err_diag.run()
                    err_diag.destroy()
                i = 0
                for ver in self.P2P.versions:
                    if ver["version"] == version:
                        del self.P2P.versions[i]
                        break
                    i = i + 1
                if self.P2P.default_winex == version:
                    print "Removed default winex"
                    if len( self.P2P.versions ):
                        print "Setting to %s" % self.P2P.versions[0]["version"]
                        self.P2P.default_winex = self.P2P.versions[0]["version"]
                    else:
                        print "Setting to None"
                        self.P2P.default_winex = "None"
                self.widgets["version_list"].set_model( self.version_model() )
                self.P2P.write()

    def default_cb( self, *args ):
        sel = self.widgets["version_list"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                 self.P2P.default_winex = model.get_value( iter, 0 )
                 new_model = self.version_model()
                 self.widgets["version_list"].set_model( new_model )
                 self.P2P.write()

    def select_cb( self, selection ):
        sel = selection.get_selected()
        if sel:
            model, iter = sel
            if iter:
                installed = model.get_value( iter, 1 )
                if installed == "Y":
                    self.widgets["install"].set_sensitive( 0 )
                    self.widgets["remove"].set_sensitive( 1 )
                    default = model.get_value( iter, 2 )
                    if default == "Y":
                        self.widgets["default"].set_sensitive( 0 )
                    else:
                        self.widgets["default"].set_sensitive( 1 )
                else:
                    self.widgets["install"].set_sensitive( 1 )
                    self.widgets["remove"].set_sensitive( 0 )
                    self.widgets["default"].set_sensitive( 0 )
        
    def run(self):
        resp = gtk.Dialog.run(self)


if __name__ == "__main__":
    import Point2Play
    if len( sys.argv ) < 3:
        print "Missing arguments"
        sys.exit()
    a = Point2Play.Point2Play( "game.ini" )
    winex_get.login_transgaming( sys.argv[1], sys.argv[2] )
    b = WinexDialog( a )
    print b.run()
